#include "displayManager.h"
#include <Arduino.h>


displayManager::displayManager(TFT_eSPI &tft) : myTFT(tft) 
{
  for (uint8_t i = 0; i < NUMBEROFDISPLAYS; i++) 
  {
    pinMode(myTFTList[i], OUTPUT);
    digitalWrite(myTFTList[i], LOW);
  }

  myTFT.init();
  myTFT.setRotation(TFT_ROTATION);
  #ifdef TFT_DMA_ENABLED
    myTFT.DMA_Enabled = TFT_DMA_ENABLED;
  #endif

  this->reset();

  #ifdef DEBUGENABLED
    Serial.println("displayManager");
    Serial.println("TFT_MOSI:" + String(TFT_MOSI));
    Serial.println("TFT_MISO:" + String(TFT_MISO));
    Serial.println("TFT_SCLK:" + String(TFT_SCLK));
    Serial.println("TFT_CS:" + String(TFT_CS));
    Serial.println("TFT_DC:" + String(TFT_DC));
    Serial.println("TFT_RST:" + String(TFT_RST));
    Serial.println("SCREEN_1_CS:" + String(SCREEN_1_CS));
    Serial.println("SCREEN_2_CS:" + String(SCREEN_2_CS));
    Serial.println("SCREEN_3_CS:" + String(SCREEN_3_CS));
  #endif
}


TFT_eSPI &displayManager::getDisplay() 
{
  return myTFT;
}


void displayManager::selectScreen(uint8_t myScreen) 
{
  if (myScreen >= NUMBEROFDISPLAYS)
    return;

  for (uint8_t i = 0; i < NUMBEROFDISPLAYS; i++) 
  {
    if (i == myScreen)     
          digitalWrite(myTFTList[i], LOW);
    else digitalWrite(myTFTList[i], HIGH);
  }
}


void displayManager::fillAllScreens(uint32_t myColor) 
{
  selectAllScreens();
  myTFT.fillScreen(myColor);
  reset();
}



void displayManager::clearAllScreens() 
{
  fillAllScreens(TFT_BACKGROUND_COLOR);
}



void displayManager::clearScreen(uint8_t myScreen) 
{
  selectScreen(myScreen);
  myTFT.fillScreen(TFT_BACKGROUND_COLOR);
}



void displayManager::selectAllScreens() 
{
  for (uint8_t i = 0; i < NUMBEROFDISPLAYS; i++) 
  {
    digitalWrite(myTFTList[i], LOW);
  }
}



void displayManager::reset() 
{
  for (uint8_t i = 0; i < NUMBEROFDISPLAYS; i++)
  {
    digitalWrite(myTFTList[i], HIGH);
  }
}