#include "main.h"
#include "displayManager.h"
#include "Button.h"
#include <TJpg_Decoder.h>
#include "icons.h"
#include <WiFi.h>
#include <HTTPClient.h>
#include <PubSubClient.h>
#include <time.h>

TFT_eSPI tft = TFT_eSPI();

Button myButtonList[NUMBEROFDISPLAYS];

struct myEntry 
{
  boolean isTasmota = false;
  boolean hasMQTTStart = false;
  boolean hasMQTTPowerDown = false;
  boolean isOff = true;
  int16_t actWatt = -1;
  boolean hasPowerDownScript = true;
  String myPowerDownMQTTTopic = "";
  String myPowerDownMQTTLoad = "";
  uint8_t standbyWatt = 4;
  String myIP = "";
  String mySwitchOff = "";
  String mySwitchOn = "";
  uint64_t lastChecked = 0;
  uint64_t lastActiveValue = 0;
  int8_t DisplayLast = DISPLAY_UNDEFINED;
  boolean canDisplayClock = true;
  boolean showInfoMessage = false;
  long myInfoMessageTimeStamp = 0l;
  long myInfoMessageDuration = MYINFODISPLAYTIMESECONDS * 1000;
  boolean showAlarmMessage = false;
  boolean showAlarmText = false;
  boolean showAlarmIcon = false;
};


myEntry  myEntryList[NUMBEROFDISPLAYS];


WiFiClient espClient;
PubSubClient myMQTTClient(espClient); 


displayManager* myDisplayManager;

// Button states
bool lastButton1State = HIGH;
bool lastButton2State = HIGH;
bool lastButton3State = HIGH;


bool tft_output(int16_t x, int16_t y, uint16_t w, uint16_t h, uint16_t *bitmap) 
{
  TFT_eSPI myDisplay = myDisplayManager->getDisplay();

  if (y >= myDisplay.height())
    return 0;
  myDisplay.pushImage(x, y, w, h, bitmap);
  return 1;
}

//------------------------------------------- Uhr Ziffenblatt ----------------------------------------------------------------------------------------------

void createDial (int8_t myIndex)
{
  uint16_t i;

  if (myEntryList[myIndex].DisplayLast == DISPLAY_ANALOGCLOCK)
    return;
  
  if (myEntryList[myIndex].showAlarmMessage) return;

  myDisplayManager->selectScreen(myIndex);
  TFT_eSPI myDisplay = myDisplayManager->getDisplay();

  myDisplay.fillCircle(120, 120, 118, TFT_BLUE);
  myDisplay.fillCircle(120, 120, 110, TFT_BACKGROUND_COLOR);   

  // Stundenmarker
  for (i = 0; i < 360; i += 30)                                                     
  {                                                   
    sx = cos((i-90) * DEG2RAD);
    sy = sin((i-90) * DEG2RAD);
    x0 = sx * 114 + 120;
    yy0 = sy * 114 + 120;
    x1 = sx * 100 + 120;
    yy1 = sy * 100 + 120;
    myDisplay.drawLine(x0, yy0, x1, yy1, TFT_WHITE);
  }
                                                             
  // Minuten Punkte
  for (i = 0; i < 360; i += 6)                                                      
  {
    sx = cos((i-90) * DEG2RAD);
    sy = sin((i-90) * DEG2RAD);
    x0 = sx * 102 + 120;
    yy0 = sy * 102 + 120;    
    myDisplay.drawPixel(x0, yy0, TFT_WHITE);
    
    if(i == 0  || i == 180) myDisplay.fillCircle (x0, yy0, 2, TFT_WHITE);                         
    if(i == 90 || i == 270) myDisplay.fillCircle (x0, yy0, 2, TFT_WHITE);
  }
  
  myDisplay.fillCircle(120, 121, 3, TFT_WHITE);                                     
  targetTime = millis() + 1000; 
  myEntryList[myIndex].DisplayLast = DISPLAY_ANALOGCLOCK;  
}

//----------------------------------------------------------------------------------------------------------------------------------------------------------

boolean getInternetTime()
{
  if (!WiFi.isConnected())
    connectToWifi();

  struct tm myTimeInfo;

  if(getLocalTime(&myTimeInfo))
  {
    myAnalogClockHour = myTimeInfo.tm_hour;
    myAnalogClockMinute = myTimeInfo.tm_min;
    myAnalogClockSecond = myTimeInfo.tm_sec;
    if (myTimeInfo.tm_mday > 9)
      strcpy(myAnalogClockDateString, String(myTimeInfo.tm_mday).c_str());
    else
    {
      myAnalogClockDateString[0] = ' ';
      strcpy(&myAnalogClockDateString[1], String(myTimeInfo.tm_mday).c_str());
    }
    myAnalogClockDateString[2] = 0;

    #ifdef DEBUGENABLED
      Serial.printf("getInternetTime() Uhrzeit: %i:%i.%i\n\r", myAnalogClockHour, myAnalogClockMinute, myAnalogClockSecond);
      Serial.printf("getInternetTime() Datum Tag: %s\n\r", myAnalogClockDateString);
    #endif
    return(true);
  }
  else
  {
    #ifdef DEBUGENABLED
      Serial.printf("getInternetTime() Fehler getLocalTime()\n\r");
    #endif
    myAnalogClockHour = 0;
    myAnalogClockMinute = 0;
    myAnalogClockSecond = 0;
    strcpy(myAnalogClockDateString, "  ");
    return(false);
  }
}

//-------------------------------------------------------------------------------------------------------------------------------------

void showAnalogClockTime(int8_t myIndex)
{
  if (myEntryList[myIndex].showAlarmMessage) return;

  if (targetTime < millis())
  {
    myDisplayManager->selectScreen(myIndex);

    TFT_eSPI myDisplay = myDisplayManager->getDisplay();
    
    targetTime += 1000;

    myAnalogClockSecond++;                                                                            

    if (myAnalogClockSecond >= 60)
    {
      myAnalogClockSecond = 0;
      myAnalogClockMinute++;  
      if(myAnalogClockMinute > 59)
      {        
        myAnalogClockMinute=0;
        createDial(myIndex);
        myAnalogClockHour++;
        if (myAnalogClockHour > 24)
        {
          myAnalogClockHour=0;
          strcpy(myAnalogClockDateString, "  ");
        }
      }
      
      if ((myAnalogClockMinute % 5) == 0)
      {
        if (getInternetTime() == false)
        {
          myEntryList[myIndex].lastActiveValue = millis();
          myDisplayManager->selectScreen(myIndex);
          TJpgDec.drawJpg(0, 0, powerbtn_off, sizeof(powerbtn_off));
          myEntryList[myIndex].DisplayLast = DISPLAY_POWERBTN;
          return;
        }
      }
    }
      
    sdeg = myAnalogClockSecond*6;                                                                     
    mdeg = myAnalogClockMinute*6+sdeg*0.01666667;                                                     
    hdeg = myAnalogClockHour*30+mdeg*0.0833333;                                                     
    hx = cos ((hdeg-90)*DEG2RAD);    
    hy = sin ((hdeg-90)*DEG2RAD);
    mx = cos ((mdeg-90)*DEG2RAD);    
    my = sin ((mdeg-90)*DEG2RAD);
    sx = cos ((sdeg-90)*DEG2RAD);    
    sy = sin ((sdeg-90)*DEG2RAD);

    if (myAnalogClockSecond==0 || firstStartAnalogClock) 
    {
      firstStartAnalogClock = false;
      myDisplay.drawLine (ohx, ohy, 120, 121, TFT_BACKGROUND_COLOR);                                     
      ohx = hx * 62 + 121;    
      ohy = hy * 62 + 121;
      myDisplay.drawLine (omx, omy, 120, 121, TFT_BACKGROUND_COLOR);
      omx = mx * 84 + 120;    
      omy = my * 84 + 121;
    }
 
    myDisplay.drawLine (osx, osy, 120, 121, TFT_BACKGROUND_COLOR);                                      
    osx = sx * 90 + 121;    
    osy = sy * 90 + 121;
    
    myDisplay.setTextFont(0);

    if (String(myAnalogClockDateString).compareTo(String(myAnalogClockOldDateString)) != 0)
    {
      myDisplay.setTextColor(TFT_BACKGROUND_COLOR);
      myDisplay.drawString(myAnalogClockOldDateString, 188, 112, 4);
      strcpy(myAnalogClockOldDateString, myAnalogClockDateString);
    }
    myDisplay.setTextColor(TFT_ORANGE);
    myDisplay.drawString(myAnalogClockDateString, 188, 112, 4);

    myDisplay.drawLine (osx, osy, 120, 121, TFT_RED);
    myDisplay.drawLine (ohx, ohy, 120, 121, TFT_WHITE);
    myDisplay.drawLine (omx, omy, 120, 121, TFT_WHITE);
    myDisplay.drawLine (osx, osy, 120, 121, TFT_RED);
    myDisplay.fillCircle(120, 121, 3, TFT_RED);
  }
}

//-------------------------------------------------------------------------------------------------------------------------------------------------

int getFreeInfoScreen()
{
  uint8_t i;
  for (i=0; i < NUMBEROFDISPLAYS; i++)
  {
    if ((myEntryList[i].DisplayLast == DISPLAY_UNDEFINED) || (myEntryList[i].DisplayLast == DISPLAY_POWERBTN) || (myEntryList[i].DisplayLast == DISPLAY_ANALOGCLOCK))
    {
      return(i);
    }
  }
  for (i=0; i < NUMBEROFDISPLAYS; i++)
  {
    if ((myEntryList[i].DisplayLast == DISPLAY_WATTAGE))
      return(i);
  }
  return (-1);
}

//-------------------------------------------------------------------------------------------------------------------------------------------------

String httpGETRequest(const char* myServerName) 
{
  WiFiClient myClient;
  HTTPClient myHTTPClient;
    
  myHTTPClient.begin(myClient, myServerName);
  
  int httpResponseCode = myHTTPClient.GET();

  String myPayload = "";
  
  if (httpResponseCode > 0) 
  {
    #ifdef DEBUGENABLED
      Serial.printf("httpGETRequest HTTP Response [%s] code: ", myServerName);
      Serial.println(httpResponseCode);
    #endif 
    myPayload = myHTTPClient.getString();
  }
  else 
  {
    #ifdef DEBUGENABLED
      Serial.printf("httpGETRequest HTTP Response [%s] ErrorCode: %i\n\r", myServerName, httpResponseCode);
    #endif 
  }

  myHTTPClient.end();

  return myPayload;
}

//-------------------------------------------------------------------------------------------------------------------------------------------------

void readTasmotaData(uint8_t index)
{
  if (index < NUMBEROFDISPLAYS)
  {
     if (myEntryList[index].isTasmota)
     {
        
        myEntryList[index].actWatt = -1;
        String abc = httpGETRequest(myEntryList[index].myIP.c_str());

        #ifdef DEBUGENABLED
          Serial.printf("HTTP Request an %s\n\r", myEntryList[index].myIP.c_str());
          Serial.printf("Antwort: %s\n\r", abc.c_str());
        #endif
      
        if (abc.indexOf("\"Power\":") > 0)
        {
           abc = abc.substring(abc.indexOf("\"Power\":")+8);
           if (abc.indexOf(",") > 0)
           {
              abc = abc.substring(0, abc.indexOf(","));             
           }
           myEntryList[index].actWatt = (int16_t) abc.toInt();
        }
        else myEntryList[index].actWatt = -1;
     }
  }  
}


void showAlarm(uint8_t myIndex, String myMessage1, String myMessage2)
{
  uint8_t i;

  for (i=0; i < NUMBEROFDISPLAYS; i++)
  {
    myEntryList[i].DisplayLast = DISPLAY_ALARM;
    myEntryList[i].showAlarmMessage = true;
    if (myEntryList[i].showAlarmIcon)
    {
      myDisplayManager->selectScreen(i); 
      TFT_eSPI myDisplay = myDisplayManager->getDisplay();
      TJpgDec.drawJpg(0, 0, alarm_icon, sizeof(alarm_icon));
    }
    else
    {
      myDisplayManager->selectScreen(i);
      TFT_eSPI myDisplay = myDisplayManager->getDisplay();
      myDisplay.fillScreen(TFT_RED);
      myDisplay.fillCircle(120,120, 102, TFT_BLACK);

      myDisplay.setTextFont(0);  
      myDisplay.setTextColor(TFT_WHITE);
  
      myDisplay.drawCentreString(myMessage1.c_str(), 120, 80, 4);
      myDisplay.drawCentreString(myMessage2.c_str(), 120, 140, 4);
    }
  }
}

//---------------------------------------------------------------------------------------

void doMQTTPowerDown(int myIndex)
{
  if ((myEntryList[myIndex].myPowerDownMQTTTopic.length() > 0) && (myEntryList[myIndex].myPowerDownMQTTLoad.length() > 0))
  {
    #ifdef DEBUGENABLED
      Serial.printf("doMQTTPowerDown myIndex: %i\n\r", myIndex);
    #endif
  
    MQTTReconnect();
    myMQTTClient.publish(myEntryList[myIndex].myPowerDownMQTTTopic.c_str(), myEntryList[myIndex].myPowerDownMQTTLoad.c_str());
    #ifdef DEBUGENABLED
      Serial.printf("doMQTTPowerDown publish Topic: %s, Load:%s\n\r", myEntryList[myIndex].myPowerDownMQTTTopic.c_str(), myEntryList[myIndex].myPowerDownMQTTLoad.c_str());
    #endif
  }
}

//--------------------------------------------------------------------------------------

boolean doTasmotaSwitchOff(int myIndex)
{
  String myResult = httpGETRequest(myEntryList[myIndex].mySwitchOff.c_str());
  #ifdef DEBUGENABLED
    Serial.printf("doTasmotaSwitchOff[%i] Request: %s\n\r", myIndex, myEntryList[myIndex].mySwitchOff.c_str());
    Serial.printf("doTasmotaSwitchOff[%i] Result: %s\n\r", myIndex, myResult.c_str());
  #endif
  if (myResult.compareTo("{\"POWER\":\"OFF\"}"))
       return true;
  else return false;
}

//---------------------------------------------------------------------

boolean doTasmotaSwitchOn(int myIndex)
{
  String myResult = httpGETRequest(myEntryList[myIndex].mySwitchOn.c_str());
  #ifdef DEBUGENABLED
    Serial.printf("doTasmotaSwitchOn[%i] Request: %s\n\r", myIndex, myEntryList[myIndex].mySwitchOff.c_str());
    Serial.printf("doTasmotaSwitchOn[%i] Result: %s\n\r", myIndex, myResult.c_str());
  #endif
  if (myResult.compareTo("{\"POWER\":\"ON\"}"))
       return true;
  else return false;
}

//---------------------------------------------------------------------------------

void connectToWifi()
{
  uint16_t i;

  for (i = 0; i < NUMBEROFDISPLAYS; i++)
    myEntryList[i].DisplayLast = DISPLAY_WIFICONNECT;

  uint8_t myActiveDisplay = 0;

  String myProgress = "#";
  myDisplayManager->clearAllScreens();
  myDisplayManager->fillAllScreens(TFT_BLACK);
  myDisplayManager->selectScreen(myActiveDisplay);
  myDisplayManager->getDisplay().setTextFont(0);        
  myDisplayManager->getDisplay().setTextColor(TFT_WHITE);
  myDisplayManager->getDisplay().drawCentreString("WIFI", 120, 90, 4);
  myDisplayManager->getDisplay().drawCentreString("Connect", 120, 130, 4);
  if (NUMBEROFDISPLAYS > 0)
       myActiveDisplay++;
  else delay(2000);
  
  myDisplayManager->selectScreen(myActiveDisplay);
  myDisplayManager->getDisplay().drawCentreString("SSID", 120, 90, 4);
  myDisplayManager->getDisplay().drawCentreString(WIFI_SSID, 120, 130, 4);
  
  WiFi.setAutoConnect(true);
  WiFi.enableSTA(true);
  
  if (NUMBEROFDISPLAYS > 1)
       myActiveDisplay++;
  else delay(2000);

  myDisplayManager->selectScreen(myActiveDisplay);
  myDisplayManager->getDisplay().setTextColor(TFT_GREENYELLOW);
  
  WiFi.begin(WIFI_SSID, WIFI_PASS);

  i = 0;
  myProgress = "";

  while(WiFi.status() != WL_CONNECTED)
  {
    myDisplayManager->getDisplay().drawCentreString(myProgress, 120, 110, 4);
    myProgress += "#";
    if (myProgress.length() > 10)
    {
      myProgress = "";
      myDisplayManager->clearScreen(myActiveDisplay);
    }
    delay(100);
    i++;
    if (i > 100)
    {
      myDisplayManager->clearAllScreens();
      myDisplayManager->selectScreen(myActiveDisplay);
      myDisplayManager->getDisplay().setTextColor(TFT_RED);
      myDisplayManager->getDisplay().drawCentreString("Kein", 120, 80, 4);
      myDisplayManager->getDisplay().drawCentreString("WiFi Connect", 120, 120, 4);
      myDisplayManager->getDisplay().drawCentreString("Starte neu", 120, 160, 4);
      delay(3500);
      myDisplayManager->clearAllScreens();
      esp_restart();
    }
  }

  #ifdef NTP_SERVER
    configTime(gmtOffset_sec, daylightOffset_sec, NTP_SERVER);
  #endif
}

//---------------------------------------------------------------------------------------------------------------------------------------------

void MQTTcallback(char* myTopic, byte* myPlayload, uint32_t myPayloadLength) 
{
  String myMessageTotal = "";
  String myLine1 = "";
  String myLine2 = "";
  #ifdef DEBUGENABLED
    Serial.print("MQTT Callback: [");
  #endif
  for (int i=0; i < myPayloadLength; i++) 
  {
    myMessageTotal += String( (char)myPlayload[i]);
  }
  #ifdef DEBUGENABLED
    Serial.printf("%s]\n\r", myMessageTotal.c_str());
  #endif  
  if (myMessageTotal.indexOf("\"line1\": \"") > 0)
  {
      myLine1 = myMessageTotal.substring(myMessageTotal.indexOf("\"line1\": \"")+10);
      myLine1.remove(myLine1.indexOf("\""));
  }
  if (myMessageTotal.indexOf("\"line2\": \"") > 0)
  {
      myLine2 = myMessageTotal.substring(myMessageTotal.indexOf("\"line2\": \"")+10);
      myLine2.remove(myLine2.indexOf("\""));
  }
  #ifdef DEBUGENABLED
    Serial.printf("MQTT Payload [%s],[%s]\n\r", myLine1.c_str(), myLine2.c_str());
  #endif
  showAlarm(2, myLine1, myLine2);
}

//----------------------------------------------------------------------------------------------------------------------------------------------------

void doSwitchOff(uint8_t myIndex)
{
  if (myIndex < NUMBEROFDISPLAYS)
  {
    if ((myEntryList[myIndex].actWatt > myEntryList[myIndex].standbyWatt) && (myEntryList[myIndex].hasMQTTPowerDown))
    {
      doMQTTPowerDown(myIndex);
      return;
    }
    else doTasmotaSwitchOff(myIndex);
  }
}

//--------------------------------------------------------------------------------------------------------------------------------------------------------------

void MQTTReconnect()
{
  if (myMQTTClient.connected())
    return;

  int myInfoScreen = getFreeInfoScreen();
  if (myInfoScreen >= 0)
    showInfoMessage(myInfoScreen, "MQTT Reconnect");

  while (!myMQTTClient.connected())
  {
    myMQTTClient.setServer(MQTTSERVER, MQTTSERVERPORT);  
    myMQTTClient.setCallback(MQTTcallback);
    myMQTTClient.connect(MQTTCLIENTID, MQTTUSER, MQTTPASSWORD);  
    #ifdef DEBUGENABLED
      Serial.println("MQTT (Re)connect....");
    #endif
    delay(250);
    if (!myMQTTClient.connected()) delay(2500);
  }
  String myString = String(MQTTCLIENTID) + "/AlarmInfo";
  myMQTTClient.publish(myString.c_str(), "Hallo");
  myString = String(MQTTCLIENTID) + "/AlarmInfo";
  #ifdef DEBUGENABLED
    Serial.printf("MQTT Connection: %i\n\r", myMQTTClient.connected());
    Serial.printf("MQTT Topic subscribed: %s\n\r", myString.c_str());
  #endif
  myMQTTClient.subscribe(myString.c_str());
}



void setup() 
{
  setCpuFrequencyMhz(80);


  #ifdef DEBUGENABLED
    Serial.begin(115200);
    Serial.println();
    Serial.println("Starte....");
  #endif


  for (int i=0; i < NUMBEROFDISPLAYS; i++)
  {
    if (i==0) myButtonList[0].beginWithPin(BUTTON_1);
    if (i==1) myButtonList[1].beginWithPin(BUTTON_2);
    if (i==2) myButtonList[2].beginWithPin(BUTTON_3);
  }


  myDisplayManager = new displayManager(tft);
  myDisplayManager->selectAllScreens();
  myDisplayManager->getDisplay().fillScreen(TFT_BLACK);
  myDisplayManager->reset();

  TJpgDec.setSwapBytes(true); // jpeg rendering setup
  TJpgDec.setCallback(tft_output);

  connectToWifi();
  MQTTReconnect();

  myDisplayManager->clearAllScreens();
  myDisplayManager->selectAllScreens();
  TJpgDec.drawJpg(0, 0, powerbtn_off, sizeof(powerbtn_off));

  for (uint8_t i = 0; i < NUMBEROFDISPLAYS; i++)
    myEntryList[i].DisplayLast = DISPLAY_POWERBTN;

  myEntryList[0].myIP = "http://192.168.0.53/cm?cmnd=status%2010";
  myEntryList[0].mySwitchOff = "http://192.168.0.53/cm?cmnd=Power%20Off";
  myEntryList[0].mySwitchOn = "http://192.168.0.53/cm?cmnd=Power%20On";
  myEntryList[0].isTasmota = true;
  myEntryList[0].hasMQTTPowerDown = true;
  myEntryList[0].myPowerDownMQTTTopic = "homeassistant/button/FANLESS-8700G-satellite/FANLESS-8700G-satellite_shutdown/set";
  myEntryList[0].myPowerDownMQTTLoad = "PRESS";
  myEntryList[0].showAlarmIcon = true;


  myEntryList[1].myIP = "http://192.168.0.60/cm?cmnd=status%2010";
  myEntryList[1].mySwitchOff = "http://192.168.0.60/cm?cmnd=Power%20Off";
  myEntryList[1].mySwitchOn = "http://192.168.0.60/cm?cmnd=Power%20On";
  myEntryList[1].isTasmota = true;
  myEntryList[1].hasMQTTPowerDown = true;
  myEntryList[1].myPowerDownMQTTTopic = "homeassistant/button/-satellite_shutdown/set";
  myEntryList[1].myPowerDownMQTTLoad = "PRESS";
  myEntryList[1].canDisplayClock = true;

  myEntryList[2].showAlarmIcon = true;

  for (int i=NUMBEROFDISPLAYS-1; i >= 0; i--)
    myEntryList[i].lastActiveValue = millis();

}



void updateDisplay(uint8_t myIndex)
{
  if (myIndex >= NUMBEROFDISPLAYS) 
    return;

  if (myEntryList[myIndex].showInfoMessage)
    return;

  if (myEntryList[myIndex].showAlarmMessage)
    return;

  myDisplayManager->selectScreen(myIndex);
  TFT_eSPI myDisplay = myDisplayManager->getDisplay();

  if (myEntryList[myIndex].actWatt > 0)
  {
    if (myEntryList[myIndex].DisplayLast != DISPLAY_WATTAGE)
    {
      myDisplay.fillScreen(TFT_DARKCYAN);
      myDisplay.setTextColor(TFT_BLACK);
      myDisplay.setTextFont(1);
      if ((myEntryList[myIndex].hasMQTTPowerDown) && 
          (myEntryList[myIndex].actWatt > myEntryList[myIndex].standbyWatt))
           myDisplay.drawCentreString("MQTT OFF", 120, 188, 4);
      else myDisplay.drawCentreString("Power OFF", 120, 188, 4);
      myEntryList[myIndex].DisplayLast = DISPLAY_WATTAGE;
    }
    myDisplay.fillRect(0,36,240,140, TFT_BLACK);
    myDisplay.setTextFont(0);
    myDisplay.setTextColor(TFT_WHITE);
    String abc = String(myEntryList[myIndex].actWatt);
    myDisplay.drawCentreString(abc.c_str(), 120, 55, 8);
    myDisplay.drawCentreString("Watt", 120, 140, 4);
    myEntryList[myIndex].lastActiveValue = millis();
  }
  else if (myEntryList[myIndex].actWatt <= 0)
  {
    if (myEntryList[myIndex].DisplayLast != DISPLAY_ANALOGCLOCK)
    {
      if (myEntryList[myIndex].DisplayLast != DISPLAY_POWERBTN)
      {
        TJpgDec.drawJpg(0, 0, powerbtn_off, sizeof(powerbtn_off));
        myEntryList[myIndex].DisplayLast = DISPLAY_POWERBTN;
        myEntryList[myIndex].lastActiveValue = millis();        
      }
    } 
  }
}

//------------------------------------------------------------------------------------------------------------------------------------------

boolean isClockDisplayed()
{
  for (uint8_t i=0; i < NUMBEROFDISPLAYS; i++)
  {
    if (myEntryList[i].DisplayLast == DISPLAY_ANALOGCLOCK) 
      return true;
  }
  return false;
}

//------------------------------------------------------------------------------------------------------------------------------------------

void showInfoMessage(int8_t myIndex, String myMessage)
{
  if (myIndex >= NUMBEROFDISPLAYS)
    return;
  
  if (myEntryList[myIndex].showAlarmMessage)
    return;

  if (myMessage == NULL)
    myMessage = "";

  myDisplayManager->selectScreen(myIndex);
  TFT_eSPI myDisplay = myDisplayManager->getDisplay();
  myEntryList[myIndex].DisplayLast = DISPLAY_INFO;
  
  myDisplay.fillScreen(TFT_GREENYELLOW);
  myDisplay.setTextColor(TFT_BLACK);
  myDisplay.setTextFont(1);
  myDisplay.drawCentreString("INFO", 120, 200, 4);
  myDisplay.drawCentreString("INFO", 120, 16, 4);

  myDisplay.fillRect(0,50,240,140, TFT_BLACK);
  myDisplay.setTextFont(0);
  myDisplay.setTextColor(TFT_WHITE);

  myDisplay.drawCentreString(myMessage.c_str(), 120, 95, 4);
  myDisplay.drawCentreString("Bitte warten...", 120, 135, 2);
  myEntryList[myIndex].showInfoMessage = true;
  myEntryList[myIndex].myInfoMessageTimeStamp = millis();
}

//----------------------------------------------------------------------------------------------------------------------------------------------

void clearInfoMessages()
{
  for (int i=0; i < NUMBEROFDISPLAYS; i++)
  {
    if (myEntryList[i].showInfoMessage)
    {
      if (millis()-myEntryList[i].myInfoMessageTimeStamp > myEntryList[i].myInfoMessageDuration)
      {
        myEntryList[i].showInfoMessage = false;
        myEntryList[i].DisplayLast = DISPLAY_UNDEFINED;
      }
    }
  }
}

//-------------------------------------------------------------------------------------------------------------------------------------------------------------

void loop() 
{
  int16_t i;
  boolean myResult = false;

  boolean myClockDisplayed = isClockDisplayed();


  if (!WiFi.isConnected())
    connectToWifi();

  if (myMQTTClient.connected())
       myMQTTClient.loop();
  else MQTTReconnect();

  if (!myEntryList[0].showAlarmMessage)
  {
    clearInfoMessages();

    for (i=0; i < NUMBEROFDISPLAYS; i++)
    {
      if (millis()-myEntryList[i].lastChecked > (myCheckIntervalSeconds * 1000) )
      {
        myEntryList[i].lastActiveValue = myEntryList[i].actWatt;
        readTasmotaData(i);
        updateDisplay(i);
        myEntryList[i].lastChecked = millis();
    
        if (!myClockDisplayed)
        {
          if (myEntryList[i].canDisplayClock)
          {
            #ifdef DEBUGENABLED
              Serial.printf("Kein Uhr angezeigt Checke Index %i %i\n\r", i, myEntryList[i].lastActiveValue);
            #endif
            if ((millis() - myEntryList[i].lastActiveValue > (1000 * myAnalogSecondsTillDisplayed)) && (!myEntryList[i].showInfoMessage) && (!myEntryList[i].showAlarmMessage))
            {
              if (getInternetTime())
              {
                createDial (i);
                showAnalogClockTime(i);
                myClockDisplayed = true;
              } 
              else myEntryList[i].lastActiveValue = millis();   // Uhrzeit Hole Fehler.....  probiere spaeter wieder
            }
          }
        }
      }
    }

    for (int i=0; i < NUMBEROFDISPLAYS; i++ )
    {
      if (myEntryList[i].actWatt > 0)
        myEntryList[i].lastActiveValue = millis();
    }
  }


  if (myButtonList[0].pressed())
  {
    if (myEntryList[0].showAlarmMessage)
    {
      for (int i=0; i < NUMBEROFDISPLAYS; i++)
        myEntryList[i].showAlarmMessage = false;
      return;
    }
    myDisplayManager->selectScreen(0);
    TJpgDec.drawJpg(0, 0, powerbtn_pressed, sizeof(powerbtn_pressed));
    myEntryList[0].DisplayLast = DISPLAY_POWERACTIVEBTN;
    if (myEntryList[0].actWatt > 0)
    {
      doSwitchOff(0);
      showInfoMessage(0, "Power OFF");
    }
    else if (myEntryList[0].actWatt == 0)
    {
      doTasmotaSwitchOn(0);
      showInfoMessage(0, "Power ON");
    }
    myEntryList[0].lastActiveValue = millis();
    updateDisplay(0);
  }

  if (myButtonList[1].pressed())
  {
    if (myEntryList[1].showAlarmMessage)
    {
      for (int i=0; i < NUMBEROFDISPLAYS; i++)
        myEntryList[i].showAlarmMessage = false;
      return;
    }
    myDisplayManager->selectScreen(1);
    TJpgDec.drawJpg(0, 0, powerbtn_pressed, sizeof(powerbtn_pressed));
    myEntryList[1].DisplayLast = DISPLAY_POWERACTIVEBTN;
    if (myEntryList[1].actWatt > 0)
    {
      doSwitchOff(1);
      showInfoMessage(1, "Power OFF");
    }
    else if (myEntryList[1].actWatt == 0)
    {
      doTasmotaSwitchOn(1);
      showInfoMessage(1, "Power ON");
    }
    myEntryList[1].lastActiveValue = millis();
    updateDisplay(1);
  }
 

  //--------------------------------------------------------------------------------------------------------------------------

  if (myButtonList[2].pressed())
  {
    if (myButtonList[2].hasBeenPressed == false)
    {
       myButtonList[2].hasBeenPressed = true;
    }
    else
    {
      myButtonList[2].hasBeenPressed = false;
      showInfoMessage(2, "DOPPELKLICK");
      #ifdef DEBUGENABLED
        Serial.printf("Button 3 - DoubleClick\n\r");
      #endif
    }
  }

  if (myButtonList[2].hasBeenPressed && (myButtonList[2].getlastAction() > BUTTON_DOUBLECLICK_TIME))
  {
    showInfoMessage(2, "SINGLE CLICK");
    #ifdef DEBUGENABLED
      Serial.printf("Button 3 - SinglePress\n\r");
    #endif 
    myButtonList[2].hasBeenPressed = false;
  }

  //------------------------------------------------------------------------------------------------------------------------------

  if (myClockDisplayed)
  {
    for (i=0; i < NUMBEROFDISPLAYS; i++)
    {
      if ((myEntryList[i].DisplayLast == DISPLAY_ANALOGCLOCK) && (!myEntryList[i].showInfoMessage))
        showAnalogClockTime(i);
    }
  }
}